@@ -65,24 +65,39 @@ _poppler_document_new_from_pdfdoc (PDFDoc  *newDoc,
                                    GError **error)
 {
   PopplerDocument *document;
-  int err;
 
   document = (PopplerDocument *) g_object_new (POPPLER_TYPE_DOCUMENT, NULL, NULL);
 
   if (!newDoc->isOk()) {
-    err = newDoc->getErrorCode();
-    delete newDoc;
-    if (err == errEncrypted) {
-      g_set_error (error, POPPLER_ERROR,
-		   POPPLER_ERROR_ENCRYPTED,
-		   "Document is encrypted.");
-    } else {
-      g_set_error (error, G_FILE_ERROR,
-		   G_FILE_ERROR_FAILED,
-		   "Failed to load document from data (error %d)'\n",
-		   err);
+    switch (newDoc->getErrorCode())
+      {
+      case errOpenFile:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_OPEN_FILE,
+		     "Failed to open the PDF file");
+	break;
+      case errBadCatalog:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_BAD_CATALOG,
+		     "Failed to read the document catalog");
+	break;
+      case errDamaged:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_DAMAGED,
+		     "PDF document is damaged");
+	break;
+      case errEncrypted:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_ENCRYPTED,
+		     "Document is encrypted");
+	break;
+      default:
+        g_set_error (error, POPPLER_ERROR,
+		     POPPLER_ERROR_INVALID,
+		     "Failed to load document");
     }
-
+    
+    delete newDoc;
     return NULL;
   }
 
@@ -122,7 +137,6 @@ poppler_document_new_from_file (const char  *uri,
   PDFDoc *newDoc;
   GooString *filename_g;
   GooString *password_g;
-  int err;
   char *filename;
 
   if (!globalParams) {
@@ -153,8 +167,7 @@ poppler_document_new_from_file (const char  *uri,
   }
 
   newDoc = new PDFDoc(filename_g, password_g, password_g);
-  if (password_g)
-    delete password_g;
+  delete password_g;
 
   return _poppler_document_new_from_pdfdoc (newDoc, error);
 }
@@ -182,8 +195,6 @@ poppler_document_new_from_data (char        *data,
   PDFDoc *newDoc;
   MemStream *str;
   GooString *password_g;
-  int err;
-  char *filename;
 
   if (!globalParams) {
     globalParams = new GlobalParams();
@@ -198,8 +209,7 @@ poppler_document_new_from_data (char        *data,
     password_g = new GooString (password);
 
   newDoc = new PDFDoc(str, password_g, password_g);
-  if (password_g)
-    delete password_g;
+  delete password_g;
 
   return _poppler_document_new_from_pdfdoc (newDoc, error);
 }
@@ -210,7 +220,9 @@ poppler_document_new_from_data (char        *data,
  * @uri: uri of file to save
  * @error: return location for an error, or %NULL
  * 
- * Saves @document.  If @error is set, %FALSE will be returned. Possible errors
+ * Saves @document. Any change made in the document such as 
+ * form fields filled by the user will be saved. 
+ * If @error is set, %FALSE will be returned. Possible errors
  * include those in the #G_FILE_ERROR domain.
  * 
  * Return value: %TRUE, if the document was successfully saved
@@ -238,6 +250,43 @@ poppler_document_save (PopplerDocument  *document,
   return retval;
 }
 
+/**
+ * poppler_document_save_a_copy:
+ * @document: a #PopplerDocument
+ * @uri: uri of file to save
+ * @error: return location for an error, or %NULL
+ * 
+ * Saves a copy of the original @document.
+ * Any change made in the document such as 
+ * form fields filled by the user will not be saved. 
+ * If @error is set, %FALSE will be returned. Possible errors
+ * include those in the #G_FILE_ERROR domain.
+ * 
+ * Return value: %TRUE, if the document was successfully saved
+ **/
+gboolean
+poppler_document_save_a_copy (PopplerDocument  *document,
+			      const char       *uri,
+			      GError          **error)
+{
+  char *filename;
+  gboolean retval = FALSE;
+
+  g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), FALSE);
+
+  filename = g_filename_from_uri (uri, NULL, error);
+  if (filename != NULL) {
+    GooString *fname = new GooString (filename);
+    g_free (filename);
+
+    retval = document->doc->saveWithoutChangesAs (fname);
+
+    delete fname;
+  }
+
+  return retval;
+}
+
 static void
 poppler_document_finalize (GObject *object)
 {
@@ -637,6 +686,8 @@ poppler_document_get_property (GObject    *object,
 	flag |= POPPLER_PERMISSIONS_OK_TO_COPY;
       if (document->doc->okToAddNotes ())
 	flag |= POPPLER_PERMISSIONS_OK_TO_ADD_NOTES;
+      if (document->doc->okToFillForm ())
+        flag |= POPPLER_PERMISSIONS_OK_TO_FILL_FORM;
       g_value_set_flags (value, flag);
       break;
     case PROP_METADATA:
