@@ -8,6 +8,7 @@
 #include "poppler-action.h"
 static const GEnumValue _poppler_action_type_values[] = {
   { POPPLER_ACTION_UNKNOWN, "POPPLER_ACTION_UNKNOWN", "unknown" },
+  { POPPLER_ACTION_NONE, "POPPLER_ACTION_NONE", "none" },
   { POPPLER_ACTION_GOTO_DEST, "POPPLER_ACTION_GOTO_DEST", "goto-dest" },
   { POPPLER_ACTION_GOTO_REMOTE, "POPPLER_ACTION_GOTO_REMOTE", "goto-remote" },
   { POPPLER_ACTION_LAUNCH, "POPPLER_ACTION_LAUNCH", "launch" },
@@ -155,6 +156,7 @@ static const GFlagsValue _poppler_permissions_values[] = {
   { POPPLER_PERMISSIONS_OK_TO_MODIFY, "POPPLER_PERMISSIONS_OK_TO_MODIFY", "ok-to-modify" },
   { POPPLER_PERMISSIONS_OK_TO_COPY, "POPPLER_PERMISSIONS_OK_TO_COPY", "ok-to-copy" },
   { POPPLER_PERMISSIONS_OK_TO_ADD_NOTES, "POPPLER_PERMISSIONS_OK_TO_ADD_NOTES", "ok-to-add-notes" },
+  { POPPLER_PERMISSIONS_OK_TO_FILL_FORM, "POPPLER_PERMISSIONS_OK_TO_FILL_FORM", "ok-to-fill-form" },
   { POPPLER_PERMISSIONS_FULL, "POPPLER_PERMISSIONS_FULL", "full" },
   { 0, NULL, NULL }
 };
@@ -247,11 +249,181 @@ poppler_form_choice_type_get_type (void)
 }
 
 
+/* enumerations from "poppler-annot.h" */
+#include "poppler-annot.h"
+static const GEnumValue _poppler_annot_type_values[] = {
+  { POPPLER_ANNOT_UNKNOWN, "POPPLER_ANNOT_UNKNOWN", "unknown" },
+  { POPPLER_ANNOT_TEXT, "POPPLER_ANNOT_TEXT", "text" },
+  { POPPLER_ANNOT_LINK, "POPPLER_ANNOT_LINK", "link" },
+  { POPPLER_ANNOT_FREE_TEXT, "POPPLER_ANNOT_FREE_TEXT", "free-text" },
+  { POPPLER_ANNOT_LINE, "POPPLER_ANNOT_LINE", "line" },
+  { POPPLER_ANNOT_SQUARE, "POPPLER_ANNOT_SQUARE", "square" },
+  { POPPLER_ANNOT_CIRCLE, "POPPLER_ANNOT_CIRCLE", "circle" },
+  { POPPLER_ANNOT_POLYGON, "POPPLER_ANNOT_POLYGON", "polygon" },
+  { POPPLER_ANNOT_POLY_LINE, "POPPLER_ANNOT_POLY_LINE", "poly-line" },
+  { POPPLER_ANNOT_HIGHLIGHT, "POPPLER_ANNOT_HIGHLIGHT", "highlight" },
+  { POPPLER_ANNOT_UNDERLINE, "POPPLER_ANNOT_UNDERLINE", "underline" },
+  { POPPLER_ANNOT_SQUIGGLY, "POPPLER_ANNOT_SQUIGGLY", "squiggly" },
+  { POPPLER_ANNOT_STRIKE_OUT, "POPPLER_ANNOT_STRIKE_OUT", "strike-out" },
+  { POPPLER_ANNOT_STAMP, "POPPLER_ANNOT_STAMP", "stamp" },
+  { POPPLER_ANNOT_CARET, "POPPLER_ANNOT_CARET", "caret" },
+  { POPPLER_ANNOT_INK, "POPPLER_ANNOT_INK", "ink" },
+  { POPPLER_ANNOT_POPUP, "POPPLER_ANNOT_POPUP", "popup" },
+  { POPPLER_ANNOT_FILE_ATTACHMENT, "POPPLER_ANNOT_FILE_ATTACHMENT", "file-attachment" },
+  { POPPLER_ANNOT_SOUND, "POPPLER_ANNOT_SOUND", "sound" },
+  { POPPLER_ANNOT_MOVIE, "POPPLER_ANNOT_MOVIE", "movie" },
+  { POPPLER_ANNOT_WIDGET, "POPPLER_ANNOT_WIDGET", "widget" },
+  { POPPLER_ANNOT_SCREEN, "POPPLER_ANNOT_SCREEN", "screen" },
+  { POPPLER_ANNOT_PRINTER_MARK, "POPPLER_ANNOT_PRINTER_MARK", "printer-mark" },
+  { POPPLER_ANNOT_TRAP_NET, "POPPLER_ANNOT_TRAP_NET", "trap-net" },
+  { POPPLER_ANNOT_WATERMARK, "POPPLER_ANNOT_WATERMARK", "watermark" },
+  { POPPLER_ANNOT_3D, "POPPLER_ANNOT_3D", "3d" },
+  { 0, NULL, NULL }
+};
+
+GType
+poppler_annot_type_get_type (void)
+{
+  static GType type = 0;
+
+  if (G_UNLIKELY (type == 0))
+    type = g_enum_register_static ("PopplerAnnotType", _poppler_annot_type_values);
+
+  return type;
+}
+
+static const GFlagsValue _poppler_annot_flag_values[] = {
+  { POPPLER_ANNOT_FLAG_UNKNOWN, "POPPLER_ANNOT_FLAG_UNKNOWN", "unknown" },
+  { POPPLER_ANNOT_FLAG_INVISIBLE, "POPPLER_ANNOT_FLAG_INVISIBLE", "invisible" },
+  { POPPLER_ANNOT_FLAG_HIDDEN, "POPPLER_ANNOT_FLAG_HIDDEN", "hidden" },
+  { POPPLER_ANNOT_FLAG_PRINT, "POPPLER_ANNOT_FLAG_PRINT", "print" },
+  { POPPLER_ANNOT_FLAG_NO_ZOOM, "POPPLER_ANNOT_FLAG_NO_ZOOM", "no-zoom" },
+  { POPPLER_ANNOT_FLAG_NO_ROTATE, "POPPLER_ANNOT_FLAG_NO_ROTATE", "no-rotate" },
+  { POPPLER_ANNOT_FLAG_NO_VIEW, "POPPLER_ANNOT_FLAG_NO_VIEW", "no-view" },
+  { POPPLER_ANNOT_FLAG_READ_ONLY, "POPPLER_ANNOT_FLAG_READ_ONLY", "read-only" },
+  { POPPLER_ANNOT_FLAG_LOCKED, "POPPLER_ANNOT_FLAG_LOCKED", "locked" },
+  { POPPLER_ANNOT_FLAG_TOGGLE_NO_VIEW, "POPPLER_ANNOT_FLAG_TOGGLE_NO_VIEW", "toggle-no-view" },
+  { POPPLER_ANNOT_FLAG_LOCKED_CONTENTS, "POPPLER_ANNOT_FLAG_LOCKED_CONTENTS", "locked-contents" },
+  { 0, NULL, NULL }
+};
+
+GType
+poppler_annot_flag_get_type (void)
+{
+  static GType type = 0;
+
+  if (G_UNLIKELY (type == 0))
+    type = g_flags_register_static ("PopplerAnnotFlag", _poppler_annot_flag_values);
+
+  return type;
+}
+
+static const GEnumValue _poppler_annot_markup_reply_type_values[] = {
+  { POPPLER_ANNOT_MARKUP_REPLY_TYPE_R, "POPPLER_ANNOT_MARKUP_REPLY_TYPE_R", "r" },
+  { POPPLER_ANNOT_MARKUP_REPLY_TYPE_GROUP, "POPPLER_ANNOT_MARKUP_REPLY_TYPE_GROUP", "group" },
+  { 0, NULL, NULL }
+};
+
+GType
+poppler_annot_markup_reply_type_get_type (void)
+{
+  static GType type = 0;
+
+  if (G_UNLIKELY (type == 0))
+    type = g_enum_register_static ("PopplerAnnotMarkupReplyType", _poppler_annot_markup_reply_type_values);
+
+  return type;
+}
+
+static const GEnumValue _poppler_annot_external_data_type_values[] = {
+  { POPPLER_ANNOT_EXTERNAL_DATA_MARKUP_3D, "POPPLER_ANNOT_EXTERNAL_DATA_MARKUP_3D", "3d" },
+  { POPPLER_ANNOT_EXTERNAL_DATA_MARKUP_UNKNOWN, "POPPLER_ANNOT_EXTERNAL_DATA_MARKUP_UNKNOWN", "unknown" },
+  { 0, NULL, NULL }
+};
+
+GType
+poppler_annot_external_data_type_get_type (void)
+{
+  static GType type = 0;
+
+  if (G_UNLIKELY (type == 0))
+    type = g_enum_register_static ("PopplerAnnotExternalDataType", _poppler_annot_external_data_type_values);
+
+  return type;
+}
+
+static const GEnumValue _poppler_annot_text_icon_values[] = {
+  { POPPLER_ANNOT_TEXT_ICON_COMMENT, "POPPLER_ANNOT_TEXT_ICON_COMMENT", "comment" },
+  { POPPLER_ANNOT_TEXT_ICON_KEY, "POPPLER_ANNOT_TEXT_ICON_KEY", "key" },
+  { POPPLER_ANNOT_TEXT_ICON_NOTE, "POPPLER_ANNOT_TEXT_ICON_NOTE", "note" },
+  { POPPLER_ANNOT_TEXT_ICON_HELP, "POPPLER_ANNOT_TEXT_ICON_HELP", "help" },
+  { POPPLER_ANNOT_TEXT_ICON_NEW_PARAGRAPH, "POPPLER_ANNOT_TEXT_ICON_NEW_PARAGRAPH", "new-paragraph" },
+  { POPPLER_ANNOT_TEXT_ICON_PARAGRAPH, "POPPLER_ANNOT_TEXT_ICON_PARAGRAPH", "paragraph" },
+  { POPPLER_ANNOT_TEXT_ICON_INSERT, "POPPLER_ANNOT_TEXT_ICON_INSERT", "insert" },
+  { 0, NULL, NULL }
+};
+
+GType
+poppler_annot_text_icon_get_type (void)
+{
+  static GType type = 0;
+
+  if (G_UNLIKELY (type == 0))
+    type = g_enum_register_static ("PopplerAnnotTextIcon", _poppler_annot_text_icon_values);
+
+  return type;
+}
+
+static const GEnumValue _poppler_annot_text_state_values[] = {
+  { POPPLER_ANNOT_TEXT_STATE_MARKED, "POPPLER_ANNOT_TEXT_STATE_MARKED", "marked" },
+  { POPPLER_ANNOT_TEXT_STATE_UNMARKED, "POPPLER_ANNOT_TEXT_STATE_UNMARKED", "unmarked" },
+  { POPPLER_ANNOT_TEXT_STATE_ACCEPTED, "POPPLER_ANNOT_TEXT_STATE_ACCEPTED", "accepted" },
+  { POPPLER_ANNOT_TEXT_STATE_REJECTED, "POPPLER_ANNOT_TEXT_STATE_REJECTED", "rejected" },
+  { POPPLER_ANNOT_TEXT_STATE_CANCELLED, "POPPLER_ANNOT_TEXT_STATE_CANCELLED", "cancelled" },
+  { POPPLER_ANNOT_TEXT_STATE_COMPLETED, "POPPLER_ANNOT_TEXT_STATE_COMPLETED", "completed" },
+  { POPPLER_ANNOT_TEXT_STATE_NONE, "POPPLER_ANNOT_TEXT_STATE_NONE", "none" },
+  { POPPLER_ANNOT_TEXT_STATE_UNKNOWN, "POPPLER_ANNOT_TEXT_STATE_UNKNOWN", "unknown" },
+  { 0, NULL, NULL }
+};
+
+GType
+poppler_annot_text_state_get_type (void)
+{
+  static GType type = 0;
+
+  if (G_UNLIKELY (type == 0))
+    type = g_enum_register_static ("PopplerAnnotTextState", _poppler_annot_text_state_values);
+
+  return type;
+}
+
+static const GEnumValue _poppler_annot_free_text_quadding_values[] = {
+  { POPPLER_ANNOT_FREE_TEXT_QUADDING_LEFT_JUSTIFIED, "POPPLER_ANNOT_FREE_TEXT_QUADDING_LEFT_JUSTIFIED", "left-justified" },
+  { POPPLER_ANNOT_FREE_TEXT_QUADDING_CENTERED, "POPPLER_ANNOT_FREE_TEXT_QUADDING_CENTERED", "centered" },
+  { POPPLER_ANNOT_FREE_TEXT_QUADDING_RIGHT_JUSTIFIED, "POPPLER_ANNOT_FREE_TEXT_QUADDING_RIGHT_JUSTIFIED", "right-justified" },
+  { 0, NULL, NULL }
+};
+
+GType
+poppler_annot_free_text_quadding_get_type (void)
+{
+  static GType type = 0;
+
+  if (G_UNLIKELY (type == 0))
+    type = g_enum_register_static ("PopplerAnnotFreeTextQuadding", _poppler_annot_free_text_quadding_values);
+
+  return type;
+}
+
+
 /* enumerations from "poppler.h" */
 #include "poppler.h"
 static const GEnumValue _poppler_error_values[] = {
   { POPPLER_ERROR_INVALID, "POPPLER_ERROR_INVALID", "invalid" },
   { POPPLER_ERROR_ENCRYPTED, "POPPLER_ERROR_ENCRYPTED", "encrypted" },
+  { POPPLER_ERROR_OPEN_FILE, "POPPLER_ERROR_OPEN_FILE", "open-file" },
+  { POPPLER_ERROR_BAD_CATALOG, "POPPLER_ERROR_BAD_CATALOG", "bad-catalog" },
+  { POPPLER_ERROR_DAMAGED, "POPPLER_ERROR_DAMAGED", "damaged" },
   { 0, NULL, NULL }
 };
 
