@@ -71,27 +71,26 @@ ArthurOutputDev::ArthurOutputDev(QPainter *painter):
 {
   m_currentBrush = QBrush(Qt::SolidPattern);
   m_fontEngine = 0;
+  m_font = 0;
+  m_image = 0;
 }
 
 ArthurOutputDev::~ArthurOutputDev()
 {
+  delete m_fontEngine;
 }
 
 void ArthurOutputDev::startDoc(XRef *xrefA) {
-  int i;
-
   xref = xrefA;
-  if (m_fontEngine) {
-    delete m_fontEngine;
-  }
+  delete m_fontEngine;
   m_fontEngine = new SplashFontEngine(
 #if HAVE_T1LIB_H
-				    globalParams->getEnableT1lib(),
+  globalParams->getEnableT1lib(),
 #endif
 #if HAVE_FREETYPE_FREETYPE_H || HAVE_FREETYPE_H
-				    globalParams->getEnableFreeType(),
+  globalParams->getEnableFreeType(),
 #endif
-				    globalParams->getAntialias());
+  globalParams->getAntialias());
 }
 
 void ArthurOutputDev::startPage(int pageNum, GfxState *state)
@@ -241,7 +240,9 @@ void ArthurOutputDev::updateStrokeOpacity(GfxState *state)
 
 void ArthurOutputDev::updateFont(GfxState *state)
 {
+#ifdef __GNUC__
 #warning fix this, probably update with updated code from SplashOutputdev
+#endif
 /*
   GfxFont *gfxFont;
   GfxFontType fontType;
@@ -497,7 +498,6 @@ void ArthurOutputDev::drawChar(GfxState *state, double x, double y,
   // fill
   if (!(render & 1)) {
     int x0, y0, xFrac, yFrac;
-    SplashGlyphBitmap glyph;
 
     x0 = static_cast<int>(floor(x1));
     xFrac = splashFloor((x1 - x0) * splashFontFraction);
@@ -516,7 +516,9 @@ void ArthurOutputDev::drawChar(GfxState *state, double x, double y,
 		       fontPath->pts[i+1].x+x0, fontPath->pts[i+1].y+y0);
 	  ++i;
 	}
+#ifdef __GNUC__
 #warning FIX THIS
+#endif
 // 	else if (fontPath->flags[i] & splashPathArcCW) {
 // 	  qDebug() << "Need to implement arc";
 // 	}
@@ -681,8 +683,7 @@ void ArthurOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
   int x, y;
   ImageStream *imgStr;
   Guchar *pix;
-  GfxRGB rgb;
-  int alpha, i;
+  int i;
   double *ctm;
   QMatrix matrix;
   int is_identity_transform;
@@ -735,6 +736,7 @@ void ArthurOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 
   if (m_image == NULL || m_image->isNull()) {
     qDebug() << "Null image";
+    delete imgStr;
     return;
   }
   ctm = state->getCTM();
@@ -742,6 +744,8 @@ void ArthurOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 
   m_painter->setMatrix(matrix, true);
   m_painter->drawImage( QPoint(0,0), *m_image );
+  delete m_image;
+  m_image = 0;
   free (buffer);
   delete imgStr;
 
