@@ -25,6 +25,7 @@
 #include "PageLabelInfo.h"
 #include "Catalog.h"
 #include "Form.h"
+#include "OptionalContent.h"
 
 //------------------------------------------------------------------------
 // Catalog
@@ -33,6 +34,7 @@
 Catalog::Catalog(XRef *xrefA) {
   Object catDict, pagesDict, pagesDictRef;
   Object obj, obj2;
+  Object optContentProps;
   char *alreadyRead;
   int numPages0;
   int i;
@@ -45,6 +47,7 @@ Catalog::Catalog(XRef *xrefA) {
   baseURI = NULL;
   pageLabelInfo = NULL;
   form = NULL;
+  optContent = NULL;
 
   xref->getCatalog(&catDict);
   if (!catDict.isDict()) {
@@ -171,6 +174,11 @@ Catalog::Catalog(XRef *xrefA) {
   // get the outline dictionary
   catDict.dictLookup("Outlines", &outline);
 
+  // get the Optional Content dictionary
+  catDict.dictLookup("OCProperties", &optContentProps);
+  optContent = new OCGs(&optContentProps, xref);
+  optContentProps.free();
+
   // perform form-related loading after all widgets have been loaded
   if (form) 
     form->postWidgetsLoad();
@@ -208,6 +216,7 @@ Catalog::~Catalog() {
   }
   delete pageLabelInfo;
   delete form;
+  delete optContent;
   metadata.free();
   structTreeRoot.free();
   outline.free();
@@ -387,6 +396,7 @@ EmbFile *Catalog::embeddedFile(int i)
     GooString *createDate = new GooString();
     GooString *modDate = new GooString();
     GooString *checksum = new GooString();
+    GooString *mimetype = new GooString();
     Stream *efStream = NULL;
     int size = -1;
     if (obj.isRef()) {
@@ -427,13 +437,13 @@ EmbFile *Catalog::embeddedFile(int i)
 		// dataDict corresponds to Table 3.41 in the PDF1.6 spec.
 		Dict *dataDict = efStream->getDict();
 
-		// subtype is normally mimetype. You can extract it with code like this:
-		// Object subtypeName;
-		// dataDict->lookup( "Subtype", &subtypeName );
-		// It is optional, so this will sometimes return a null object
-		// if (subtypeName.isName()) {
-		//        std::cout << "got subtype name: " << subtypeName.getName() << std::endl;
-		// }
+		// subtype is normally the mimetype
+		Object subtypeName;
+		if (dataDict->lookup("Subtype", &subtypeName)->isName()) {
+		    delete mimetype;
+		    mimetype = new GooString(subtypeName.getName());
+		}
+		subtypeName.free();
 
 		// paramDict corresponds to Table 3.42 in the PDF1.6 spec
 		Object paramDict;
@@ -469,7 +479,7 @@ EmbFile *Catalog::embeddedFile(int i)
 	    obj2.free();
 	}
     }
-    EmbFile *embeddedFile = new EmbFile(fileName, desc, size, createDate, modDate, checksum, strObj);
+    EmbFile *embeddedFile = new EmbFile(fileName, desc, size, createDate, modDate, checksum, mimetype, strObj);
     strObj.free();
     return embeddedFile;
 }
@@ -559,7 +569,6 @@ int NameTree::Entry::cmp(const void *voidKey, const void *voidEntry)
 GBool NameTree::lookup(GooString *name, Object *obj)
 {
   Entry **entry;
-  char *cname;
 
   entry = (Entry **) bsearch(name, entries,
 			     length, sizeof(Entry *), Entry::cmp);
