@@ -23,7 +23,6 @@
 #ifdef WIN32
 #  include <shlobj.h>
 #endif
-#include <fontconfig/fontconfig.h>
 #include "goo/gmem.h"
 #include "goo/GooString.h"
 #include "goo/GooList.h"
@@ -533,9 +532,11 @@ Plugin::~Plugin() {
 GlobalParams::GlobalParams() {
   UnicodeMap *map;
   int i;
-  
+
+#ifndef _MSC_VER  
   FcInit();
   FCcfg = FcConfigGetCurrent();
+#endif
 
 #if MULTITHREADED
   gInitMutex(&mutex);
@@ -756,10 +757,6 @@ GBool GlobalParams::parseYesNo2(char *token, GBool *flag) {
 }
 
 GlobalParams::~GlobalParams() {
-  GooHashIter *iter;
-  GooString *key;
-  GooList *list;
-
   freeBuiltinFontTables();
 
   delete macRomanReverseMap;
@@ -773,9 +770,7 @@ GlobalParams::~GlobalParams() {
   deleteGooList(toUnicodeDirs, GooString);
   deleteGooHash(displayFonts, DisplayFontParam);
 #ifdef WIN32
-  if (winFontList) {
-    delete winFontList;
-  }
+  delete winFontList;
 #endif
   deleteGooHash(psFonts, PSFontParam);
   deleteGooList(psNamedFonts16, PSFontParam);
@@ -783,8 +778,12 @@ GlobalParams::~GlobalParams() {
   delete textEncoding;
   deleteGooList(fontDirs, GooString);
 
+  GooHashIter *iter;
+  GooString *key;
   cMapDirs->startIter(&iter);
-  while (cMapDirs->getNext(&iter, &key, (void **)&list)) {
+  void *val;
+  while (cMapDirs->getNext(&iter, &key, &val)) {
+    GooList* list = (GooList*)val;
     deleteGooList(list, GooString);
   }
   delete cMapDirs;
@@ -926,12 +925,13 @@ GBool findModifier(const char *name, const char *modifier, const char **start)
   }
 }
 
+#ifndef _MSC_VER
 static FcPattern *buildFcPattern(GfxFont *font)
 {
-  int weight = FC_WEIGHT_NORMAL,
-      slant = FC_SLANT_ROMAN,
-      width = FC_WIDTH_NORMAL,
-      spacing = FC_PROPORTIONAL;
+  int weight = -1,
+      slant = -1,
+      width = -1,
+      spacing = -1;
   bool deleteFamily = false;
   char *family, *name, *lang, *modifiers;
   const char *start;
@@ -1057,17 +1057,23 @@ static FcPattern *buildFcPattern(GfxFont *font)
   
   p = FcPatternBuild(NULL,
                     FC_FAMILY, FcTypeString, family,
-                    FC_SLANT, FcTypeInteger, slant, 
-                    FC_WEIGHT, FcTypeInteger, weight,
-                    FC_WIDTH, FcTypeInteger, width, 
-                    FC_SPACING, FcTypeInteger, spacing,
                     FC_LANG, FcTypeString, lang,
                     NULL);
+  if (slant != -1) FcPatternAddInteger(p, FC_SLANT, slant);
+  if (weight != -1) FcPatternAddInteger(p, FC_WEIGHT, weight);
+  if (width != -1) FcPatternAddInteger(p, FC_WIDTH, width);
+  if (spacing != -1) FcPatternAddInteger(p, FC_SPACING, spacing);
+
   if (deleteFamily)
     delete[] family;
   return p;
 }
+#endif
 
+/* if you can't or don't want to use Fontconfig, you need to implement
+   this function for your platform. For Windows, it's in GlobalParamsWin.cc
+*/
+#ifndef _MSC_VER
 DisplayFontParam *GlobalParams::getDisplayFont(GfxFont *font) {
   DisplayFontParam *dfp;
   FcPattern *p=0;
@@ -1126,6 +1132,7 @@ fin:
   unlockGlobalParams;
   return dfp;
 }
+#endif
 
 GBool GlobalParams::getPSExpandSmaller() {
   GBool f;
@@ -1475,28 +1482,26 @@ CharCodeToUnicode *GlobalParams::getCIDToUnicode(GooString *collection) {
 }
 
 CharCodeToUnicode *GlobalParams::getUnicodeToUnicode(GooString *fontName) {
-  CharCodeToUnicode *ctu;
-  GooHashIter *iter;
-  GooString *fontPattern, *fileName;
-
   lockGlobalParams;
-  fileName = NULL;
+  GooHashIter *iter;
   unicodeToUnicodes->startIter(&iter);
-  while (unicodeToUnicodes->getNext(&iter, &fontPattern, (void **)&fileName)) {
+  GooString *fileName = NULL;
+  GooString *fontPattern;
+  void *val;
+  while (!fileName && unicodeToUnicodes->getNext(&iter, &fontPattern, &val)) {
     if (strstr(fontName->getCString(), fontPattern->getCString())) {
       unicodeToUnicodes->killIter(&iter);
-      break;
+      fileName = (GooString*)val;
     }
-    fileName = NULL;
   }
+  CharCodeToUnicode *ctu = NULL;
   if (fileName) {
-    if (!(ctu = unicodeToUnicodeCache->getCharCodeToUnicode(fileName))) {
-      if ((ctu = CharCodeToUnicode::parseUnicodeToUnicode(fileName))) {
-	unicodeToUnicodeCache->add(ctu);
-      }
+    ctu = unicodeToUnicodeCache->getCharCodeToUnicode(fileName);
+    if (!ctu) {
+      ctu = CharCodeToUnicode::parseUnicodeToUnicode(fileName);
+      if (ctu)
+         unicodeToUnicodeCache->add(ctu);
     }
-  } else {
-    ctu = NULL;
   }
   unlockGlobalParams;
   return ctu;
