@@ -30,6 +30,14 @@
 
 namespace Poppler {
 
+static GooString *QStringToGooString(const QString &s) {
+    int len = s.length();
+    char *cstring = (char *)gmallocn(s.length(), sizeof(char));
+    for (int i = 0; i < len; ++i)
+      cstring[i] = s.at(i).unicode();
+    return new GooString(cstring, len);
+}
+
 Document *Document::load(const QString &filePath)
 {
   if (!globalParams) {
@@ -70,8 +78,9 @@ bool Document::unlock(const QCString &password)
 {
   if (data->locked) {
     /* racier then it needs to be */
+    GooString *filename = new GooString(data->doc.getFileName());
     GooString *pwd = new GooString(password.data());
-    DocumentData *doc2 = new DocumentData(data->doc.getFileName(), pwd);
+    DocumentData *doc2 = new DocumentData(filename, pwd);
     delete pwd;
     if (!doc2->doc.isOk()) {
       delete doc2;
@@ -79,6 +88,7 @@ bool Document::unlock(const QCString &password)
       delete data;
       data = doc2;
       data->locked = false;
+      data->m_fontInfoScanner = new FontInfoScanner(&(data->doc));
     }
   }
   return data->locked;
@@ -132,6 +142,7 @@ bool Document::scanForFonts( int numPages, QValueList<FontInfo> *fontList ) cons
                   (Poppler::FontInfo::Type)((::FontInfo*)items->get(i))->getType());
     fontList->append(font);
   }
+  deleteGooList(items, ::FontInfo);
   return true;
 }
 
@@ -206,7 +217,6 @@ QDateTime Document::getDate( const QString & type ) const
   }
 
   Object obj;
-  char *s;
   int year, mon, day, hour, min, sec;
   Dict *infoDict = info.getDict();
   QString result;
