@@ -27,6 +27,10 @@
 #include "poppler-link.h"
 #include "poppler-annotation-private.h"
 
+// poppler includes
+#include <Page.h>
+#include <Annot.h>
+
 namespace Poppler {
 
 //BEGIN AnnotationUtils implementation
@@ -669,19 +673,19 @@ class LineAnnotationPrivate : public AnnotationPrivate
         QLinkedList<QPointF> linePoints;
         LineAnnotation::TermStyle lineStartStyle;
         LineAnnotation::TermStyle lineEndStyle;
-        bool lineClosed;  // (if true draw close shape)
+        bool lineClosed : 1;  // (if true draw close shape)
+        bool lineShowCaption : 1;
         QColor lineInnerColor;
         double lineLeadingFwdPt;
         double lineLeadingBackPt;
-        bool lineShowCaption;
         LineAnnotation::LineIntent lineIntent;
 };
 
 LineAnnotationPrivate::LineAnnotationPrivate()
     : AnnotationPrivate(), lineStartStyle( LineAnnotation::None ),
     lineEndStyle( LineAnnotation::None ), lineClosed( false ),
-    lineLeadingFwdPt( 0 ), lineLeadingBackPt( 0 ),
-    lineShowCaption( false ), lineIntent( LineAnnotation::Unknown )
+    lineShowCaption( false ), lineLeadingFwdPt( 0 ), lineLeadingBackPt( 0 ),
+    lineIntent( LineAnnotation::Unknown )
 {
 }
 
@@ -785,6 +789,7 @@ void LineAnnotation::store( QDomNode & node, QDomDocument & document ) const
             lineElement.appendChild( pElement );
             pElement.setAttribute( "x", p.x() );
             pElement.setAttribute( "y", p.y() );
+            ++it;
         }
     }
 }
@@ -1442,6 +1447,7 @@ LinkAnnotation::LinkAnnotation( const QDomNode & node )
                 {
                     Poppler::LinkAction::ActionType act;
                     QString actString = qe.attribute( "action" );
+                    bool found = true;
                     if ( actString == "PageFirst" )
                         act = Poppler::LinkAction::PageFirst;
                     else if ( actString == "PagePrev" )
@@ -1466,14 +1472,21 @@ LinkAnnotation::LinkAnnotation( const QDomNode & node )
                         act = Poppler::LinkAction::GoToPage;
                     else if ( actString == "Close" )
                         act = Poppler::LinkAction::Close;
-                    Poppler::LinkAction * action = new Poppler::LinkAction( QRect(), act );
-                    d->linkDestination = action;
+                    else
+                        found = false;
+                    if (found)
+                    {
+                        Poppler::LinkAction * action = new Poppler::LinkAction( QRect(), act );
+                        d->linkDestination = action;
+                    }
                 }
+#if 0
                 else if ( type == "Movie" )
                 {
                     Poppler::LinkMovie * movie = new Poppler::LinkMovie( QRect() );
                     d->linkDestination = movie;
                 }
+#endif
             }
         }
 
@@ -1590,10 +1603,14 @@ void LinkAnnotation::store( QDomNode & node, QDomDocument & document ) const
             }
             case Poppler::Link::Movie:
             {
-                Poppler::LinkMovie * movie = static_cast< Poppler::LinkMovie * >( d->linkDestination );
                 hyperlinkElement.setAttribute( "type", "Movie" );
                 break;
             }
+            case Poppler::Link::Sound:
+            {
+                // FIXME: implement me
+                break;
+            }
             case Poppler::Link::None:
                 break;
         }
@@ -1648,4 +1665,31 @@ void LinkAnnotation::setLinkRegionPoint( int id, const QPointF &point )
     d->linkRegion[id] = point;
 }
 
+//BEGIN utility annotation functions
+QColor convertAnnotColor( AnnotColor *color )
+{
+    if ( !color )
+        return QColor();
+
+    QColor newcolor;
+    double *color_data = color->getValues();
+    switch ( color->getSpace() )
+    {
+        case AnnotColor::colorTransparent: // = 0,
+            newcolor = Qt::transparent;
+            break;
+        case AnnotColor::colorGray: // = 1,
+            newcolor.setRgbF( color_data[0], color_data[0], color_data[0] );
+            break;
+        case AnnotColor::colorRGB: // = 3,
+            newcolor.setRgbF( color_data[0], color_data[1], color_data[2] );
+            break;
+        case AnnotColor::colorCMYK: // = 4
+            newcolor.setCmykF( color_data[0], color_data[1], color_data[2], color_data[3] );
+            break;
+    }
+    return newcolor;
+}
+//END utility annotation functions
+
 }
