@@ -29,7 +29,6 @@
 
 #include <QtCore/QDebug>
 #include <QtCore/QFile>
-#include <QtGui/QImage>
 #include <QtCore/QByteArray>
 
 #include "poppler-private.h"
@@ -44,7 +43,7 @@ namespace Poppler {
 	DocumentData *doc = new DocumentData(new GooString(QFile::encodeName(filePath)), 
 					     new GooString(ownerPassword.data()),
 					     new GooString(userPassword.data()));
-	return checkDocument(doc);
+	return DocumentData::checkDocument(doc);
     }
 
     Document *Document::loadFromData(const QByteArray &fileContents,
@@ -55,10 +54,10 @@ namespace Poppler {
 	DocumentData *doc = new DocumentData(fileContents,
 					     new GooString(ownerPassword.data()),
 					     new GooString(userPassword.data()));
-	return checkDocument(doc);
+	return DocumentData::checkDocument(doc);
     }
     
-    Document *Document::checkDocument(DocumentData *doc)
+    Document *DocumentData::checkDocument(DocumentData *doc)
     {
 	Document *pdoc;
 	if (doc->doc->isOk() || doc->doc->getErrorCode() == errEncrypted) {
@@ -91,7 +90,7 @@ namespace Poppler {
 
     Page *Document::page(int index) const
     {
-	return new Page(this, index);
+	return new Page(m_doc, index);
     }
 
     bool Document::isLocked() const
@@ -129,7 +128,7 @@ namespace Poppler {
 	return m_doc->locked;
     }
 
-    Document::PageMode Document::pageMode(void) const
+    Document::PageMode Document::pageMode() const
     {
 	switch (m_doc->doc->getCatalog()->getPageMode()) {
 	case Catalog::pageModeNone:
@@ -149,7 +148,7 @@ namespace Poppler {
 	}
     }
 
-    Document::PageLayout Document::pageLayout(void) const
+    Document::PageLayout Document::pageLayout() const
     {
 	switch (m_doc->doc->getCatalog()->getPageLayout()) {
 	case Catalog::pageLayoutNone:
@@ -209,54 +208,28 @@ namespace Poppler {
 	// [Albert] Code adapted from pdfinfo.cc on xpdf
 	Object info;
 	if ( m_doc->locked )
-	    return NULL;
+	    return QString();
 
 	m_doc->doc->getDocInfo( &info );
 	if ( !info.isDict() )
-	    return NULL;
+	    return QString();
 
 	QString result;
 	Object obj;
 	GooString *s1;
-	GBool isUnicode;
-	Unicode u;
-	int i;
 	Dict *infoDict = info.getDict();
 
 	if ( infoDict->lookup( type.toLatin1().data(), &obj )->isString() )
 	{
 	    s1 = obj.getString();
-	    if ( ( s1->getChar(0) & 0xff ) == 0xfe && ( s1->getChar(1) & 0xff ) == 0xff )
-	    {
-		isUnicode = gTrue;
-		i = 2;
-	    }
-	    else
-	    {
-		isUnicode = gFalse;
-		i = 0;
-	    }
-	    while ( i < obj.getString()->getLength() )
-	    {
-		if ( isUnicode )
-		{
-		    u = ( ( s1->getChar(i) & 0xff ) << 8 ) | ( s1->getChar(i+1) & 0xff );
-		    i += 2;
-		}
-		else
-		{
-		    u = s1->getChar(i) & 0xff;
-		    ++i;
-		}
-		result += unicodeToQString( &u, 1 );
-	    }
+	    result = UnicodeParsedString(s1);
 	    obj.free();
 	    info.free();
 	    return result;
 	}
 	obj.free();
 	info.free();
-	return NULL;
+	return QString();
     }
 
     QStringList Document::infoKeys() const
@@ -474,6 +447,11 @@ namespace Poppler {
         return new PSConverter(m_doc);
     }
 
+    PDFConverter *Document::pdfConverter() const
+    {
+        return new PDFConverter(m_doc);
+    }
+
     QString Document::metadata() const
     {
         QString result;
@@ -487,6 +465,19 @@ namespace Poppler {
         return result;
     }
 
+    bool Document::hasOptionalContent() const
+    {
+        return ( m_doc->doc->getOptContentConfig()->hasOCGs() );
+    }
+
+    OptContentModel *Document::optionalContentModel()
+    {
+        if (m_doc->m_optContentModel.isNull()) {
+	    m_doc->m_optContentModel = new OptContentModel(m_doc->doc->getOptContentConfig(), 0);
+	}
+        return (OptContentModel *)m_doc->m_optContentModel;
+    }
+
     QDateTime convertDate( char *dateString )
     {
         int year;
@@ -499,9 +490,12 @@ namespace Poppler {
         int tzHours = 0;
         int tzMins = 0;
 
+        if ( dateString == NULL ) return QDateTime();
+        if ( strlen(dateString) < 2 ) return QDateTime();
+
         if ( dateString[0] == 'D' && dateString[1] == ':' )
             dateString += 2;
-        if ( sscanf( dateString,
+        if ( dateString != NULL && sscanf( dateString,
 		     "%4d%2d%2d%2d%2d%2d%c%2d%*c%2d",
 		     &year, &mon, &day, &hour, &min, &sec,
 		     &tz, &tzHours, &tzMins ) > 0 ) {
