@@ -22,14 +22,14 @@
 
 #include "poppler-annotation.h"
 #include "poppler-link.h"
+#include "poppler-optcontent.h"
 #include "poppler-page-transition.h"
 
 #include <QtCore/QByteArray>
 #include <QtCore/QDateTime>
 #include <QtCore/QSet>
-#include <QtCore/QVector>
-#include <QtGui/QPixmap>
 #include <QtXml/QDomDocument>
+#include "poppler-export.h"
 
 class EmbFile;
 class Sound;
@@ -40,6 +40,7 @@ class Sound;
 namespace Poppler {
 
     class Document;
+    class DocumentData;
 
     class PageData;
 
@@ -47,6 +48,7 @@ namespace Poppler {
 
     class TextBoxData;
 
+    class PDFConverter;
     class PSConverter;
 
     /**
@@ -58,7 +60,7 @@ namespace Poppler {
 	- a QRectF that gives a box that describes where on the page
 	the text is found.
     */
-    class TextBox {
+    class POPPLER_QT4_EXPORT TextBox {
     friend class Page;
     public:
       /**
@@ -83,10 +85,21 @@ namespace Poppler {
       */
       QRectF boundingBox() const;
 
+      /**
+	  Returns the pointer to the next text box, if there is one.
+
+	  Otherwise, it returns a null pointer.
+      */
       TextBox *nextWord() const;
 
-      double edge(int i) const;
+      /**
+	  Returns the bounding box of the \p i -th characted of the word.
+      */
+      QRectF charBoundingBox(int i) const;
 
+      /**
+	  Returns whether there is a space character after this text box
+      */
       bool hasSpaceAfter() const;
 
     private:
@@ -101,8 +114,11 @@ namespace Poppler {
        Container class for information about a font within a PDF
        document
     */
-    class FontInfo {
+    class POPPLER_QT4_EXPORT FontInfo {
     public:
+	/**
+	   The type of font.
+	*/
 	enum Type {
 		unknown,
 		Type1,
@@ -118,10 +134,12 @@ namespace Poppler {
 		CIDTrueTypeOT
 	};
 	
+	/// \cond PRIVATE
 	/**
 	   Create a new font information container.
 	*/
 	FontInfo( const FontInfoData &fid );
+	/// \endcond
 	
 	/**
 	   Copy constructor.
@@ -192,7 +210,7 @@ namespace Poppler {
     /**
        Container class for an embedded file with a PDF document
     */
-    class EmbeddedFile {
+    class POPPLER_QT4_EXPORT EmbeddedFile {
     public:
 	/// \cond PRIVATE
 	EmbeddedFile(EmbFile *embfile);
@@ -233,12 +251,17 @@ namespace Poppler {
 	QDateTime createDate() const;
 	
 	/**
-		The checksum of the file.
+	   The MD5 checksum of the file.
 		
-		This will return an empty QString if there is no checksum element
+	   This will return an empty QByteArray if there is no checksum element.
 	*/
 	QByteArray checksum() const;
 
+	/**
+	   The MIME type of the file, if known.
+	*/
+	QString mimeType() const;
+
 	/**
 	   The data as a byte array
 	*/
@@ -259,7 +282,7 @@ namespace Poppler {
     /**
        Page within a PDF document
     */
-    class Page {
+    class POPPLER_QT4_EXPORT Page {
 	friend class Document;
     public:
 	/**
@@ -453,19 +476,18 @@ namespace Poppler {
     private:
 	Q_DISABLE_COPY(Page)
 
-	Page(const Document *doc, int index);
+	Page(DocumentData *doc, int index);
 	PageData *m_page;
     };
 
-    class DocumentData;
-
 /**
    PDF document
 
    A document potentially contains multiple Pages
 */
-    class Document {
+    class POPPLER_QT4_EXPORT Document {
 	friend class Page;
+	friend class DocumentData;
   
     public:
 	/**
@@ -852,11 +874,37 @@ QString subject = m_doc->info("Subject");
 	 */
 	PSConverter *psConverter() const;
 	
+	/**
+	  Gets a new PDF converter for this document.
+
+	  The caller gets the ownership of the returned converter.
+	 */
+	PDFConverter *pdfConverter() const;
+	
 	/**
 	  Gets the metadata stream contents
 	*/
 	QString metadata() const;
 
+	/**
+	   Test whether this document has "optional content".
+
+	   Optional content is used to optionally turn on (display)
+	   and turn off (not display) some elements of the document.
+	   The most common use of this is for layers in design
+	   applications, but it can be used for a range of things,
+	   such as not including some content in printing, and
+	   displaying content in the appropriate language.
+	*/
+	bool hasOptionalContent() const;
+
+	/**
+	   Itemviews model for optional content.
+
+	   The model is owned by the document.
+	*/
+	OptContentModel *optionalContentModel();
+
 	/**
 	   Destructor.
 	*/
@@ -868,10 +916,49 @@ QString subject = m_doc->info("Subject");
 	DocumentData *m_doc;
 	
 	Document(DocumentData *dataA);
-	static Document *checkDocument(DocumentData *doc);
     };
     
-    class PSConverterData;
+    class BaseConverterPrivate;
+    class PSConverterPrivate;
+    class PDFConverterPrivate;
+    /**
+       \brief Base converter.
+
+       This is the base class for the converters.
+    */
+    class POPPLER_QT4_EXPORT BaseConverter
+    {
+        friend class Document;
+        public:
+            /**
+              Destructor.
+            */
+            virtual ~BaseConverter();
+
+            /** Sets the output file name. You must set this or the output device. */
+            void setOutputFileName(const QString &outputFileName);
+
+            /** Sets the output device. You must set this or the output file name. */
+            void setOutputDevice(QIODevice *device);
+
+            /**
+              Does the conversion.
+
+              \return whether the conversion succeeded
+            */
+            virtual bool convert() = 0;
+
+        protected:
+            /// \cond PRIVATE
+            BaseConverter(BaseConverterPrivate &dd);
+            Q_DECLARE_PRIVATE(BaseConverter)
+            BaseConverterPrivate *d_ptr;
+            /// \endcond
+
+        private:
+            Q_DISABLE_COPY(BaseConverter)
+    };
+
     /**
        Converts a PDF to PS
 
@@ -886,7 +973,7 @@ width = dummy.width();
 height = dummy.height();
        \endcode
     */
-    class PSConverter
+    class POPPLER_QT4_EXPORT PSConverter : public BaseConverter
     {
         friend class Document;
         public:
@@ -895,9 +982,6 @@ height = dummy.height();
             */
             ~PSConverter();
 
-            /** Sets the output file name. Mandatory. */
-            void setOutputFileName(const QString &outputFileName);
-
             /** Sets the list of pages to print. Mandatory. */
             void setPageList(const QList<int> &pageList);
 
@@ -963,25 +1047,57 @@ height = dummy.height();
             /** Defines if the page will be rasterized to an image before printing. Defaults to false */
             void setForceRasterize(bool forceRasterize);
 
-            /**
-              Does the conversion.
-
-              \return whether the conversion succeeded
-            */
             bool convert();
 
         private:
+            Q_DECLARE_PRIVATE(PSConverter)
             Q_DISABLE_COPY(PSConverter)
 
             PSConverter(DocumentData *document);
+    };
 
-            PSConverterData *m_data;
+    /**
+       Converts a PDF to PDF (thus saves a copy of the document).
+    */
+    class POPPLER_QT4_EXPORT PDFConverter : public BaseConverter
+    {
+        friend class Document;
+        public:
+            /**
+              Options for the PDF export.
+             */
+            enum PDFOption {
+                WithChanges = 0x00000001        ///< The changes done to the document are saved as well
+            };
+            Q_DECLARE_FLAGS( PDFOptions, PDFOption )
+
+            /**
+              Destructor.
+            */
+            virtual ~PDFConverter();
+
+            /**
+              Sets the options for the PDF export.
+             */
+            void setPDFOptions(PDFOptions options);
+            /**
+              The currently set options for the PDF export.
+             */
+            PDFOptions pdfOptions() const;
+
+            bool convert();
+
+        private:
+            Q_DECLARE_PRIVATE(PDFConverter)
+            Q_DISABLE_COPY(PDFConverter)
+
+            PDFConverter(DocumentData *document);
     };
 
     /**
        Conversion from PDF date string format to QDateTime
     */
-    QDateTime convertDate( char *dateString );
+    POPPLER_QT4_EXPORT QDateTime convertDate( char *dateString );
 
     class SoundData;
     /**
@@ -991,7 +1107,7 @@ height = dummy.height();
        whose url is represented by url() ), or Embedded, and the player has to
        play the data contained in data().
     */
-    class SoundObject {
+    class POPPLER_QT4_EXPORT SoundObject {
     public:
 	/**
 	   The type of sound
@@ -1060,4 +1176,7 @@ height = dummy.height();
 
 }
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(Poppler::Document::RenderHints)
+Q_DECLARE_OPERATORS_FOR_FLAGS(Poppler::PDFConverter::PDFOptions)
+
 #endif
