@@ -59,6 +59,7 @@ static GooString* basename(GooString* str){
   return new GooString(str);
 }
 
+#if 0
 static GooString* Dirname(GooString* str){
   
   char *p=str->getCString();
@@ -68,6 +69,7 @@ static GooString* Dirname(GooString* str){
       return new GooString(p,i+1);
   return new GooString();
 } 
+#endif
 
 //------------------------------------------------------------------------
 // HtmlString
@@ -79,8 +81,18 @@ HtmlString::HtmlString(GfxState *state, double fontSize, HtmlFontAccu* fonts) {
 
   state->transform(state->getCurX(), state->getCurY(), &x, &y);
   if ((font = state->getFont())) {
-    yMin = y - font->getAscent() * fontSize;
-    yMax = y - font->getDescent() * fontSize;
+    double ascent = font->getAscent();
+    double descent = font->getDescent();
+    if( ascent > 1.05 ){
+        //printf( "ascent=%.15g is too high, descent=%.15g\n", ascent, descent );
+        ascent = 1.05;
+    }
+    if( descent < -0.4 ){
+        //printf( "descent %.15g is too low, ascent=%.15g\n", descent, ascent );
+        descent = -0.4;
+    }
+    yMin = y - ascent * fontSize;
+    yMax = y - descent * fontSize;
     GfxRGB rgb;
     state->getFillRGB(&rgb);
     GooString *name = state->getFont()->getName();
@@ -331,6 +343,36 @@ void HtmlPage::endString() {
   curStr = NULL;
 }
 
+static const char *strrstr( const char *s, const char *ss )
+{
+  const char *p = strstr( s, ss );
+  for( const char *pp = p; pp != NULL; pp = strstr( p+1, ss ) ){
+    p = pp;
+  }
+  return p;
+}
+
+static void CloseTags( GooString *htext, GBool &finish_a, GBool &finish_italic, GBool &finish_bold )
+{
+  const char *last_italic = finish_italic && ( finish_bold   || finish_a    ) ? strrstr( htext->getCString(), "<i>" ) : NULL;
+  const char *last_bold   = finish_bold   && ( finish_italic || finish_a    ) ? strrstr( htext->getCString(), "<b>" ) : NULL;
+  const char *last_a      = finish_a      && ( finish_italic || finish_bold ) ? strrstr( htext->getCString(), "<a " ) : NULL;
+  if( finish_a && ( finish_italic || finish_bold ) && last_a > ( last_italic > last_bold ? last_italic : last_bold ) ){
+    htext->append("</a>", 4);
+    finish_a = false;
+  }
+  if( finish_italic && finish_bold && last_italic > last_bold ){
+    htext->append("</i>", 4);
+    finish_italic = false;
+  }
+  if( finish_bold )
+    htext->append("</b>", 4);
+  if( finish_italic )
+    htext->append("</i>", 4);
+  if( finish_a )
+    htext->append("</a>");
+}
+
 void HtmlPage::coalesce() {
   HtmlString *str1, *str2;
   HtmlFont *hfont1, *hfont2;
@@ -460,7 +502,7 @@ void HtmlPage::coalesce() {
 					str1->size * sizeof(double));
       if (addSpace) {
 		  str1->text[str1->len] = 0x20;
-		  str1->htext->append(" ");
+		  str1->htext->append(xml?" ":"&nbsp;");
 		  str1->xRight[str1->len] = str2->xMin;
 		  ++str1->len;
       }
@@ -492,28 +534,24 @@ void HtmlPage::coalesce() {
 	++str1->len;
       }
 
-      /* fix <i> and <b> if str1 and str2 differ */
-      if( hfont1->isBold() && !hfont2->isBold() )
-	str1->htext->append("</b>", 4);
-      if( hfont1->isItalic() && !hfont2->isItalic() )
-	str1->htext->append("</i>", 4);
-      if( !hfont1->isBold() && hfont2->isBold() )
-	str1->htext->append("<b>", 3);
-      if( !hfont1->isItalic() && hfont2->isItalic() )
-	str1->htext->append("<i>", 3);
-
-      /* now handle switch of links */
+      /* fix <i>, <b> if str1 and str2 differ and handle switch of links */
       HtmlLink *hlink1 = str1->getLink();
       HtmlLink *hlink2 = str2->getLink();
-      if( !hlink1 || !hlink2 || !hlink1->isEqualDest(*hlink2) ) {
-	if(hlink1 != NULL )
-	  str1->htext->append("</a>");
-	if(hlink2 != NULL ) {
-	  GooString *ls = hlink2->getLinkStart();
-	  str1->htext->append(ls);
-	  delete ls;
-	}
+      bool switch_links = !hlink1 || !hlink2 || !hlink1->isEqualDest(*hlink2);
+      GBool finish_a = switch_links && hlink1 != NULL;
+      GBool finish_italic = hfont1->isItalic() && ( !hfont2->isItalic() || finish_a );
+      GBool finish_bold   = hfont1->isBold()   && ( !hfont2->isBold()   || finish_a || finish_italic );
+      CloseTags( str1->htext, finish_a, finish_italic, finish_bold );
+      if( switch_links && hlink2 != NULL ) {
+        GooString *ls = hlink2->getLinkStart();
+        str1->htext->append(ls);
+        delete ls;
       }
+      if( ( !hfont1->isItalic() || finish_italic ) && hfont2->isItalic() )
+	    str1->htext->append("<i>", 3);
+      if( ( !hfont1->isBold() || finish_bold ) && hfont2->isBold() )
+	    str1->htext->append("<b>", 3);
+
 
       str1->htext->append(str2->htext);
       // str1 now contains href for link of str2 (if it is defined)
@@ -529,12 +567,10 @@ void HtmlPage::coalesce() {
       delete str2;
     } else { // keep strings separate
 //      printf("no\n"); 
-      if( hfont1->isBold() )
-	str1->htext->append("</b>",4);
-      if( hfont1->isItalic() )
-	str1->htext->append("</i>",4);
-      if(str1->getLink() != NULL )
-	str1->htext->append("</a>");
+      GBool finish_a = str1->getLink() != NULL;
+      GBool finish_bold   = hfont1->isBold();
+      GBool finish_italic = hfont1->isItalic();
+      CloseTags( str1->htext, finish_a, finish_italic, finish_bold );
      
       str1->xMin = curX; str1->yMin = curY; 
       str1 = str2;
@@ -552,12 +588,11 @@ void HtmlPage::coalesce() {
     }
   }
   str1->xMin = curX; str1->yMin = curY;
-  if( hfont1->isBold() )
-    str1->htext->append("</b>",4);
-  if( hfont1->isItalic() )
-    str1->htext->append("</i>",4);
-  if(str1->getLink() != NULL )
-    str1->htext->append("</a>");
+
+  GBool finish_bold   = hfont1->isBold();
+  GBool finish_italic = hfont1->isItalic();
+  GBool finish_a = str1->getLink() != NULL;
+  CloseTags( str1->htext, finish_a, finish_italic, finish_bold );
 
 #if 0 //~ for debugging
   for (str1 = yxStrings; str1; str1 = str1->yxNext) {
@@ -994,10 +1029,9 @@ HtmlOutputDev::~HtmlOutputDev() {
       delete pages;
 }
 
-
-
 void HtmlOutputDev::startPage(int pageNum, GfxState *state) {
-  /*if (mode&&!xml){
+#if 0
+  if (mode&&!xml){
     if (write){
       write=gFalse;
       GooString* fname=Dirname(Docname);
@@ -1012,7 +1046,8 @@ void HtmlOutputDev::startPage(int pageNum, GfxState *state) {
     // else 
       fprintf(tin,"ROTATE=%d neg %d neg translate\n",state->getX1(),state->getY1());  
     }
-  }*/
+  }
+#endif
 
   this->pageNum = pageNum;
   GooString *str=basename(Docname);
@@ -1084,8 +1119,6 @@ void HtmlOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 			      int width, int height, GBool invert,
 			      GBool inlineImg) {
 
-  int i, j;
-  
   if (ignore||complexMode) {
     OutputDev::drawImageMask(state, ref, str, width, height, invert, inlineImg);
     return;
@@ -1098,14 +1131,6 @@ void HtmlOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
   int w0, h0, w1, h1;		// size of image
   double xt, yt, wt, ht;
   GBool rotate, xFlip, yFlip;
-  GBool dither;
-  int x, y;
-  int ix, iy;
-  int px1, px2, qx, dx;
-  int py1, py2, qy, dy;
-  Gulong pixel;
-  int nComps, nVals, nBits;
-  double r1, g1, b1;
  
   // get image position and size
   state->transform(0, 0, &xt, &yt);
@@ -1175,8 +1200,6 @@ void HtmlOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
 			  int width, int height, GfxImageColorMap *colorMap,
 			  int *maskColors, GBool inlineImg) {
 
-  int i, j;
-   
   if (ignore||complexMode) {
     OutputDev::drawImage(state, ref, str, width, height, colorMap, 
 			 maskColors, inlineImg);
@@ -1184,23 +1207,12 @@ void HtmlOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
   }
 
   FILE *f1;
-  ImageStream *imgStr;
-  Guchar pixBuf[4];
-  GfxColor color;
   int c;
   
   int x0, y0;			// top left corner of image
   int w0, h0, w1, h1;		// size of image
   double xt, yt, wt, ht;
   GBool rotate, xFlip, yFlip;
-  GBool dither;
-  int x, y;
-  int ix, iy;
-  int px1, px2, qx, dx;
-  int py1, py2, qy, dy;
-  Gulong pixel;
-  int nComps, nVals, nBits;
-  double r1, g1, b1;
  
   // get image position and size
   state->transform(0, 0, &xt, &yt);
@@ -1300,10 +1312,11 @@ GooString* HtmlOutputDev::getLinkDest(Link *link,Catalog* catalog){
 	  int page=1;
 	  LinkGoTo *ha=(LinkGoTo *)link->getAction();
 	  LinkDest *dest=NULL;
-	  if (ha->getDest()==NULL) 
-	      dest=catalog->findDest(ha->getNamedDest());
-	  else 
+	  if (ha->getDest()!=NULL)
 	      dest=ha->getDest()->copy();
+	  else if (ha->getNamedDest()!=NULL)
+	      dest=catalog->findDest(ha->getNamedDest());
+	      
 	  if (dest){ 
 	      if (dest->isPageRef()){
 		  Ref pageref=dest->getPageRef();
@@ -1496,10 +1509,11 @@ GBool HtmlOutputDev::newOutlineLevel(FILE *output, Object *node, Catalog* catalo
       if (!curr.dictLookup("Dest", &dest)->isNull()) {
 		LinkGoTo *link = new LinkGoTo(&dest);
 		LinkDest *linkdest=NULL;
-		if (link->getDest()==NULL) 
-	  		linkdest=catalog->findDest(link->getNamedDest());
-		else 
-	  		linkdest=link->getDest()->copy();
+		if (link->getDest()!=NULL)
+			linkdest=link->getDest()->copy();
+		else if (link->getNamedDest()!=NULL)
+			linkdest=catalog->findDest(link->getNamedDest());
+			
 		delete link;
 		if (linkdest) { 
 	  		int page;
